#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Union, Optional, Dict, Tuple, Generator, Any

from oser.core import BitMap as _BitMap
from oser.types import OserElementType, OserStructType


class BitMap(_BitMap):
    """
    BitMap type that automatically generates :class:`oser.PaddingFlag()` at undefined bit positions.
    Reserved bits are hidden in stringified output.
    """

    def __init__(
        self, length: int, values: Optional[Union[Dict[Union[int, Tuple[int, int]], str]]] = None, strict: bool = False
    ) -> None:
        """
        Args:
            length (int): the length in bits
            values=None (dict): dictionary with key: value where key is the bit index as integer or
                a tuple of (byte, bit)
            strict=False (bool): When True, non-specified values must be 0
        """
        super(BitMap, self).__init__(length=length, values=values, strict=strict)

    def get_byte_size(self) -> int:  # type: ignore[override]
        """
        Return the size in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(BitMap, self).get_byte_size()

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data (bytes): the data buffer that is decoded.
            full_data (bytes): the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.
        """
        return super(BitMap, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data (bytes): the binary data string until the part to be encoded.
                The user normally does not need to supply this.
            context_data (bytes): the binary data of the current context.
                The user normally does not need to supply this.
        """
        return super(BitMap, self).encode(full_data=full_data, context_data=context_data)

    def from_dict(self, data: Dict) -> None:
        """
        Fill self with data.

        Args:
            data (dict): data to be used to fill the calling instance.
        """
        return super(BitMap, self).from_dict(data=data)

    def fuzzing_iterator(self, copy: bool = False) -> Generator[Any, None, None]:
        """
        The fuzzing iterator iterates over all combinations
        of set fuzzing values (:func:`oser.ByteType.set_fuzzing_values`).
        If no fuzzing values are set the current struct is yielded.

        Args:
            copy=False (bool): if set to ``True`` the generated
                fuzzing values are deep copies of the original.
                Creating these deep copies is slow. If set to ``False`` the
                original struct is retruned and the generated value must be
                used immediately since the next generated overwrites the values
                on the same value.

        Yields:
            the fuzzing combinations.
        """
        return super(BitMap, self).fuzzing_iterator(copy=copy)

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None (object): stop introspection at ``stop_at``.
        """
        return super(BitMap, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(BitMap, self).root()

    def get_size(self) -> int:
        """
        Return the size in bits.

        Returns:
            int: the length of the bit type in bits.
        """
        return super(BitMap, self).get_size()

    def to_dict(self) -> Dict:
        """
        Return self as a dict()
        """
        return super(BitMap, self).to_dict()

    def up(self) -> OserStructType:
        """
        return parent element
        """
        return super(BitMap, self).up()
