#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Union, Callable, Optional, Dict, Generator, Any, List

from oser.core import IfElse as _IfElse
from oser.types import OserElementType, OserStructType


class IfElse(_IfElse):
    """
    ``IfElse`` builds a conditional serializer for boolean conditions.
    """

    def __init__(self, condition: Union[Callable, bool], if_true: OserElementType, if_false: OserElementType) -> None:
        """
        Args:
            condition: the condition which selects the desired values. Can be a callable (e.g. lambda) or a scalar.
            if_true: is used if `condition` is `True`.
            if_false: is used if `condition` is `False`.
        """
        super(IfElse, self).__init__(condition=condition, if_true=if_true, if_false=if_false)

    def get_byte_size(self) -> int:  # type: ignore[override]
        """
        Return the current size in bytes.
        """
        return super(IfElse, self).get_byte_size()

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(IfElse, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(IfElse, self).encode(full_data=full_data, context_data=context_data)

    def from_dict(self, data: Dict) -> None:
        """
        Fill self with data.

        Args:
            data: data to be used to fill the calling instance.
        """
        return super(IfElse, self).from_dict(data=data)

    def fuzzing_iterator(self, copy: bool = False) -> Generator[Any, None, None]:
        """
        The fuzzing iterator iterates over all combinations
        of set fuzzing values (:func:`oser.ByteType.set_fuzzing_values`).
        If no fuzzing values are set the current struct is yielded.

        Args:
            copy=False: if set to ``True`` the generated
                fuzzing values are deep copies of the original.
                Creating these deep copies is slow. If set to ``False`` the
                original struct is retruned and the generated value must be
                used immediately since the next generated overwrites the values
                on the same value.

        Yields:
            the fuzzing combinations.
        """
        return super(IfElse, self).fuzzing_iterator(copy=copy)

    def get(self) -> OserElementType:
        """
        Return the current value.
        """
        return super(IfElse, self).get()

    def get_current(self) -> OserElementType:
        """
        Return the currently selected object.
        """
        return super(IfElse, self).get_current()

    def get_false_value(self) -> OserElementType:
        """
        Return the value for a ``False`` ``condition``.
        """
        return super(IfElse, self).get_false_value()

    def get_true_value(self) -> OserElementType:
        """
        Return the value for a ``True`` ``condition``.
        """
        return super(IfElse, self).get_true_value()

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(IfElse, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(IfElse, self).root()

    def set(self, value: OserElementType) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(IfElse, self).set(value=value)

    def set_false_value(self, value: OserElementType) -> None:
        """
        Set the value for a ``False`` ``condition``.

        Args:
            value: the new value for a ``False`` ``condition``.
        """
        return super(IfElse, self).set_false_value(value=value)

    def set_true_value(self, value: OserElementType) -> None:
        """
        Set the value for a ``True`` ``condition``.

        Args:
            value: the new value for a ``True`` ``condition``.
        """
        return super(IfElse, self).set_true_value(value=value)

    def get_size(self) -> int:
        """
        Return the current size in bytes or bits according to the context.
        """
        return super(IfElse, self).get_size()

    def to_dict(self) -> Union[Dict[Any, Any], List[Dict[Any, Any]]]:  # type: ignore
        """
        Return self as a dict()
        """
        return super(IfElse, self).to_dict()

    def up(self) -> OserStructType:
        """
        return parent element
        """
        return super(IfElse, self).up()
