#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Optional, Any, Union, Callable

from oser.core import JSONAdapter as _JSONAdapter
from oser.types import OserElementType, OserStructType, FuzzingValues


class JSONAdapter(_JSONAdapter):
    """
    ``JSONAdapter`` builds a JSON serializer.
    """

    def __init__(self, data: Any) -> None:
        """
        Args:
            data=None: the data to be json-ized.
        """
        super(JSONAdapter, self).__init__(data=data)

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(JSONAdapter, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(JSONAdapter, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> Any:
        """
        Return the value.

        Returns:
            the value
        """
        return super(JSONAdapter, self).get()

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(JSONAdapter, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(JSONAdapter, self).root()

    def set(self, value: Any) -> None:
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(JSONAdapter, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values: the values used for fuzzing.
        """
        return super(JSONAdapter, self).set_fuzzing_values(values=values)

    def set_length(self, length: Optional[Union[Callable, int]]) -> None:
        """
        Set the length.

        Args:
            length=None: states the string length. Can be a callable (e.g. lambda) or a scalar.
                If set to ``None`` the result is a null-terminated string.
                If set to an Integer the result is a fixed length string padded with ``padding``.
                If set to a ``callable`` (lambda or function) the result is a fixed or variable
                length string padded with ``padding``.
        """
        super(JSONAdapter, self).set_length(length=length)

    def get_size(self) -> int:
        """
        Return the size in bytes.
        """
        return super(JSONAdapter, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(JSONAdapter, self).up()
