#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Optional, Any

from oser.core import Nothing as _Nothing
from oser.types import OserElementType, OserStructType, FuzzingValues


class Nothing(_Nothing):
    """
    ``Nothing`` does not appear in the encoded data and not in the string or
    introspection.
    """

    def __init__(self) -> None:
        """
        Args:
            value: the initial value.
            format=None: Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary.
        """
        super(Nothing, self).__init__()

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bytes that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be
                decoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bytes that were decoded.
        """
        return super(Nothing, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string. This is always an empty string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(Nothing, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> None:
        """
        Always returns ``None``.

        Returns:
            None
        """
        return super(Nothing, self).get()

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(Nothing, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(Nothing, self).root()

    def set(self, value: Any) -> None:
        """
        Set the value. Always raises an Exception.

        Args:
            value: the new value
        """
        return super(Nothing, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:  # pragma: no cover
        """
        Set fuzzing values.

        Args:
            values: the values used for fuzzing.
        """
        return super(Nothing, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(Nothing, self).get_size()

    get_byte_size = get_size

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(Nothing, self).up()
