#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Optional

from oser.core import PaddingFlag as _PaddingFlag
from oser.types import OserElementType, OserStructType, FuzzingValues


class PaddingFlag(_PaddingFlag):
    """
    `PaddingFlag` is a one bit padding.
    """

    def __init__(self, value: bool = False, strict: bool = False, format: Optional[str] = None) -> None:
        """
        Args:
            value=False: can be ``True`` or ``False``.
            strict=False: if set to ``True`` an exception is raised when decoded data
                does not match ``value``.
            format=None: Use "hex" to format values to hexadecimal.
                Use "bin" to format values to binary.
        """
        super(PaddingFlag, self).__init__(value=value, strict=strict, format=format)

    def get_byte_size(self) -> int:  # type: ignore[override]
        """
        Return the length of the byte type in bytes.

        Returns:
            int: the length of the byte type in bytes.
        """
        return super(PaddingFlag, self).get_byte_size()

    def decode(self, data: bytes, full_data: bytes = b"", context_data: bytes = b"") -> int:
        """
        Decode a binary string and return the number
        of bits that were decoded.

        Args:
            data: the data buffer that is decoded.
            full_data: the binary data string until the part to be decoded.
                The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            int: the number of bits that were decoded.
        """
        return super(PaddingFlag, self).decode(data=data, full_data=full_data, context_data=context_data)

    def encode(self, full_data: bytes = b"", context_data: bytes = b"") -> bytes:
        """
        Return the encoded binary string.

        Args:
            full_data: the binary data string until the part to be
                encoded. The user normally does not need to supply this.
            context_data: the binary data of the current context.
                The user normally does not need to supply this.

        Returns:
            bytes: the encoded binary string.
        """
        return super(PaddingFlag, self).encode(full_data=full_data, context_data=context_data)

    def get(self) -> bool:
        """
        Return the value.

        Returns:
            the value.
        """
        return super(PaddingFlag, self).get()  # type: ignore

    def introspect(self, stop_at: Optional[OserElementType] = None) -> str:
        """
        Return the introspection representation of the object
        as a string.

        Args:
            stop_at=None: stop introspection at ``stop_at``.
        """
        return super(PaddingFlag, self).introspect(stop_at=stop_at)

    def root(self) -> OserStructType:
        """
        return root element
        """
        return super(PaddingFlag, self).root()

    def set(self, value: bool) -> None:  # type: ignore
        """
        Set the value.

        Args:
            value: the new value
        """
        return super(PaddingFlag, self).set(value=value)

    def set_fuzzing_values(self, values: Optional[FuzzingValues]) -> None:
        """
        Set fuzzing values.

        Args:
            values: the values used for fuzzing.
        """
        return super(PaddingFlag, self).set_fuzzing_values(values=values)

    def get_size(self) -> int:
        """
        Return the size in bits.

        Returns:
            int: the length of the bit type in bits.
        """
        return super(PaddingFlag, self).get_size()

    def up(self) -> OserStructType:
        """
        Return the parent element.
        """
        return super(PaddingFlag, self).up()
