#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Callable, Generator, Optional, List, Union

from oser.core import RandomBitsFuzzingValue as _RandomBitsFuzzingValue
from oser.core import RandomBytesFuzzingValue as _RandomBytesFuzzingValue
from oser.core import RandomFloatFuzzingValue as _RandomFloatFuzzingValue
from oser.core import RandomIntegerFuzzingValue as _RandomIntegerFuzzingValue
from oser.core import RandomStringFuzzingValue as _RandomStringFuzzingValue


def RandomBitsFuzzingValue(  # noqa: N802: this is intended
    length: int, count: int, seed: Optional[int] = None
) -> Callable[[], Generator[int, None, None]]:  # noqa: N802
    """
    A random bits fuzzing value generator.
    Random values are generated using :func:`random.getrandbits`.

    Args:
        length (int): the number of bits to be generated.
        count (int): the number of values to be generated.
        seed=None (int): the seed for :func:`random.seed`. If ``None`` is supplied no
            seed is set.

    Returns:
        callable: a callable that generates ``count`` random bits.
    """
    return _RandomBitsFuzzingValue(length=length, count=count, seed=seed)


def RandomBytesFuzzingValue(  # noqa: N802: this is intended
    length: int, count: int, choices: Optional[List[int]] = None, seed: Optional[int] = None
) -> Callable[[], Generator[bytes, None, None]]:  # noqa: N802
    """
    A random bytes fuzzing value generator.
    Random values are generated using :func:`random.choice` with ``choices``.

    Args:
        length (int): the number of bytes to be generated..
        count (int): the number of values to be generated.
        choices=None (list of int): if set to ``None`` ``list(range(256))`` is used.
        seed=None (int): the seed for :func:`random.seed`. If ``None`` is supplied no
            seed is set.

    Returns:
        callable: a callable that generates ``count`` random bytes values.
    """
    return _RandomBytesFuzzingValue(length=length, count=count, choices=choices, seed=seed)


def RandomFloatFuzzingValue(  # noqa: N802: this is intended
    factor: float, count: int, seed: Optional[Union[int, float]] = None
) -> Callable[[], Generator[float, None, None]]:  # noqa: N802
    """
    A random float fuzzing value generator.
    Random values are generated using :func:`random.random`.

    Args:
        factor (float): the factor the random value is multiplied by.
        count (int): the number of values to be generated.
        seed=None (int): the seed for :func:`random.seed`. If ``None`` is supplied no
            seed is set.

    Returns:
        callable: a callable that generates ``count`` random float values
            between ``a`` and ``b``.
    """
    return _RandomFloatFuzzingValue(factor=factor, count=count, seed=seed)


def RandomIntegerFuzzingValue(  # noqa: N802: this is intended
    a: int, b: int, count: int, seed: Optional[int] = None
) -> Callable[[], Generator[int, None, None]]:  # noqa: N802
    """
    A random integer fuzzing value generator.

    Args:
        a (int): the lower limit for :func:`random.randint`.
        b (int): the upper limit for :func:`random.randint`.
        count (int): the number of values to be generated.
        seed=None (int): the seed for :func:`random.seed`. If ``None`` is supplied no
            seed is set.

    Returns:
        callable: a callable that generates ``count`` random integer
            values between ``a`` and ``b``.
    """
    return _RandomIntegerFuzzingValue(a=a, b=b, count=count, seed=seed)


def RandomStringFuzzingValue(  # noqa: N802: this is intended
    length: int, count: int, chars: Optional[str] = None, seed: Optional[int] = None
) -> Callable[[], Generator[bytes, None, None]]:  # noqa: N802
    """
    A random string fuzzing value generator.
    Random values are generated using :func:`random.choice` with ``chars``.

    Args:
        length (int): the number of characters to be generated..
        count (int): the number of values to be generated.
        chars=None (string): if set to ``None`` ``string.printable`` is used
            (digits, letters, punctuation and whispaces).
        seed=None (int): the seed for :func:`random.seed`. If ``None`` is supplied no
            seed is set.

    Returns:
        callable: a callable that generates ``count`` random string values.
    """
    return _RandomStringFuzzingValue(length=length, count=count, chars=chars, seed=seed)
